==> squish-0.0.10.tar.gz: squish-0.0.10/rrdsquish.pm <==
package shiftarray;

no warnings "recursion";
	# setvalue over limit calls setvalue recursively

##################################################
## the object constructor (simplistic version)  ##
##################################################
sub new
{
	#my $class = shift;
	my $interval = shift;
	my $samples = shift;

	my $self = {};
	if (defined $class) {
		bless($self, $class);
	}
	else {
		bless($self);
	}

	# Length of a single sample period (e.g. 5 mins)
	$self->{interval} = $interval;	# 5 minute samples 
	
	# Number of samples to make up the period
	$self->{samples} = $samples;		

	# How long is this graph
	$self->{period} = $interval * $samples;

	# Where we start logging in the data
	$self->{offset} = 0;			
	# The data itself (nothing yet)
	$self->{data} = ();	
	
	# Never been touched
	$self->{lasttime} = 0;			

	return $self;
}

sub getperiod($)
{
	my $self=shift;
	return $self->{period};
}

sub getinterval($)
{
	my $self=shift;
	return $self->{interval};
}

sub setmaxperhour($)
{
	my $self = shift;
	my $max = shift;
	$self->{maxvalue} = $max/3600 * $self->{interval};
}

# Set a bucket value
# setvalue(x,y)
sub setvalue($$$)
{
	my $self = shift;
	my $bucket = shift;
	my $value = shift;

	if ($bucket>=$self->{samples} || $bucket<0) { return; }
	my $index=($bucket + $self->{offset} + $self->{samples}*4) % $self->{samples};

	if (defined $self->{maxvalue}) {
		if ($value < $self->{maxvalue}) {
			$self->{data}[$index]=$value;
		}
		else {
			my $max = $self->{maxvalue};
			$self->setvalue($bucket+1,$value-$max + $self->getvalue($bucket+1));
			$self->{data}[$index]=$max;
		}
	}
	else {
		$self->{data}[$index]=$value;
	}

	return $value;
}
# Get a bucket value
# y = getvalue(x);
sub getvalue($$)
{
	my $self = shift;
	my $bucket = shift;

	my $index=($bucket+$self->{offset}+$self->{samples}*4) % $self->{samples};
	my $value = $self->{data}[$index];
	if (defined $value) { return $value; } else { return 0; }
}

# Distribute $value from $a to $b
sub setvalues($$$$)
{
	my $self = shift;
	my $value = shift;
	my $a = shift;
	my $b = shift;


	my $period = $self->{period};
	my $interval = $self->{interval};
	# Maybe it's just one bucket?
	if ($a<0) {
		$value += $value / ($b-$a)*$a;
		$a=0;
	}
	if ($b < $a) { return; }
	if ($a>=$period) {
		return; # off the graph - we don't care
	}
	if ($b>$period) {
		$value -= $value / ($b-$a) * ($b-$period);
		$b=$period;
	}
	if ($b < $a) { return; }

#	# Speed -- All in the first bucket?
#	if ($a==0 && $b<=$self->{interval}) {
#		$value += $self->getvalue(0);
#		$self->setvalue(0,$value);
#		return;
#	}

	# Determine start bucket, end bucket and range betwixt
	my $first = $a / $interval;
	my $last = $b / $interval;
	my $unit;
	if ($last == $first) {
		$unit=$value;
	}
	else {
		$unit = $value / ($last-$first);  # how much is a whole bucket
	}

	# print "a:$a b:$b value:$value first:$first last:$last unit:$unit interval:$interval part:$part\n";
	$part=1-($first-int($first));
	for (my $j=int($first); $j<=$last && $value>0 ;$j++) {
		my $portion=$unit*$part;
		if ($portion>$value) { $portion=$value; }
		$self->setvalue($j, $portion + $self->getvalue($j));
		$value-=$portion;
		$part=1;
	}
}

# Move the origin a little
sub moveto($$)
{
	my $self = shift;
	my $time = shift;

	$seconds = $time - $self->{lasttime};
	if ($seconds <=0) {
		$self->{lasttime}=$time;
		return;  # can't go back in time, won't try
	}
	if ($seconds < $self->{interval}) {
		return;  # not enough to move me
	}
	if ($seconds > $self->{period}) {
		$self->{lasttime} = $time - $time % $self->{interval};
		$self->{data}=();   # all data has expired
		return;
	}
	my $shift = int($seconds / $self->{interval});
	$self->{offset} = ($self->{offset} - $shift + $self->{samples}) %
		($self->{samples});
	$self->{offset} = $self->{offset}-$shift;
	for (my $i=0; $i<$shift; $i++) {
		$self->setvalue($i,0);
	}
	$self->{lasttime} += $shift * $self->{interval};;
}

# Sum of usage from 0 seconds to $stop seconds)
# these are in terms of buckets, ie. time from origin for the last 60 seconds,
# you want sum(60)
sub sum($$)
{
	my $self=shift;
	my $stop=shift;

	my $sum=0;
	my $n=($stop/$self->{interval});
	my $ni=int($n);

	for (my $i=0; $i<$ni;$i++) {
		$sum += $self->getvalue($i);
	}
	$sum+=($n-$ni)*$self->getvalue($ni);
}

sub total($)
{
	my $self=shift;
	my $a;
	my $total=0;
	my $t = $self->{data};

	if (defined $t) {
		foreach $a (@$t) {
			if (defined $a) { $total+=$a; }
		}
	}
	return $total;
}


package rrdsquish;
#

=head1 NAME

rrdsquish - A hacky rrd, with the dubious distinction that you can modify older
data

=head1 SYNOPSIS

Store a month worth of information in a rotating database

=head1 DESCRIPTION

To run this thingy you create B<rrdsquish::new> object.  Optionally, you might
use B<rrdsquish::setmaxperhour()> so set a cieling for an hour (if you're
counting seconds, setting 3600 per hour makes sense).

Once the rrdsquish is set up, you use it in the following fashion:

=over 8

=item B<rrdsquish::moveto(origin)>

moves the origin to the timestamp given

=item B<rrdsquish::setvalues(start,stop,value)>

smears the value over the database between start and stop (inclusive) (i.e a new log entry 
was processed, applicable from B<start> to B<stop>, having a value of B<value>.

=item B<rrdsquish::sum(start,stop)>

returns the total value from start to stop (inclusive)

=back


=head1 EXAMPLE

This should do something interesting.  It sets some (additive) values into the
graph at the current time, and then rotates the graph, printing it out bit by
bit.  It's a bit silly, but it's more interesting than painting a DNS server.

  use rrdsquish;
  
  $rrd = rrdsquish::new();
  $time=1000;
  $rrd->moveto($time);
  
  $rrd->setvalues(0,59,600);
  $rrd->setvalues(30,90,600);
  foreach $a (0..60) {
  	$rrd->print(20,240);
  	sleep 2;
  	$rrd->moveto($time+$a);
  }
  $rrd->print(25,240);
  print $rrd->sum();

=head1 SEE ALSO

rrdtool -- inspiration for this travesty of data manipulation.

=head1 BUGS

At the points where resolution is lost, it is possible to accumulate excess
baggage (you have to see it to know what I mean).  It looks kinda funny on a
graph.  This is (I reckon) a problem of matching a continous system to
a discrete system, and I leave it for a higher mind to solve.  It may help to
rotate in a more synchronised manner .. hey, let's try that!

=head1 AUTHOR

Andrew McGill <andrew...wh@tever...ledge.co.za>

=cut
#'  this is to make cperl.el happy
use strict;
#use Carp;
# use vars qw(@ISA $VERSION $maxperhour);
use vars qw($VERSION $maxperhour);

$VERSION = "1.01";

sub Version { $VERSION };

# Not inherited ...
# @ISA = qw(  );

# Maximum value that the DB will show per hour
# When you add values to the DB, this is set
my $maxperhour;

# Prototypes
sub setup;


##################################################
## the object constructor (simplistic version)  ##
##################################################
sub new
{
	my $class = shift;
	my $self = {};
	my %params = @_;
	if (defined $class) {
		bless($self, $class);
	}
	else {
		bless($self);
	}
	$self->setup();
	return $self;
}

# sets up a delay list or whatever we're calling this travesty of data
# rotation and loss
sub setup
{
	my $self=shift;
	
	$self->{day}   = shiftarray::new(300,   288 );
	$self->{week}  = shiftarray::new(1800,  336 );
	$self->{month} = shiftarray::new(7200,  366 );
	$self->{year}  = shiftarray::new(86400, 366 );

	$self->{timestamp}=1;
}

# Get data for making a graph of the last 5 minutes, 5 hours, 5 weeks, etc.
# 
# ($total,$max) = getgraph($interval,$duration,\@x,\@y)
# e.g. last 60 minutes in 30 second summaries
#	getgraph(30,600,\@x,\@y)
sub getgraph($$$$)
{
	my $self = shift;
	my $tt = shift;
	my $x = shift;  # array ref
	my $y = shift;  # array ref
	my $a;
	my $total = 0;
	my $max = 0;
	my $t;

	if ($tt==0)    { $t=$self->{day}   ; }
	elsif ($tt==1) { $t=$self->{week}  ; }
	elsif ($tt==2) { $t=$self->{month} ; }
	elsif ($tt==3) { $t=$self->{year}  ; }
	else { return; }

	my $interval = $t->getinterval();
	my $i=0;
	my $xx=0;
	while ($xx < $t->{period}) {
		my $this = $$y[$i]=$t->getvalue($i);
		$total+=$this;
		if ($this > $max) { $max=$this;}
		$$x[$i]=$xx;
		$i++;
		$xx+=$t->{interval};
	}
	return ($total,$max);
}

# Shift stuff, so that the $time given is the current time for the array
# e.g. move(time());
sub moveto
{
	my $self = shift;
	my $time = shift;
	# Go through db backwards - we're assumming a loss of
	# resolution from low indexes to high .. ie. seconds, minutes,
	# hours, days, months
	$self->{day}  ->moveto($time);
	$self->{week} ->moveto($time);
	$self->{month}->moveto($time);
	$self->{year} ->moveto($time);

	$self->{timestamp}=$time;
}

# Sum of usage from 0 to $stop (excluding $stoip)
# these are in terms of buckets, ie. time from origin
# for the last 60 seconds, you want sum(0,59)
sub sum($$)
{
	my $self=shift;
	my $stop=shift;

	if ($stop <= $self->{day}  ->getperiod()) { return $self->{day}  ->sum($stop); }
	if ($stop <= $self->{week} ->getperiod()) { return $self->{week} ->sum($stop); }
	if ($stop <= $self->{month}->getperiod()) { return $self->{month}->sum($stop); }
	                                            return $self->{year} ->sum($stop);
}

# Add up a day, week, month or year
sub total($$)
{
	my $self=shift;
	my $tt=shift;
	my $t;

	if ($tt==0)    { $t=$self->{day}   ; }
	elsif ($tt==1) { $t=$self->{week}  ; }
	elsif ($tt==2) { $t=$self->{month} ; }
	elsif ($tt==3) { $t=$self->{year}  ; }
	else { return; }
	return $t->total();
}

# Print an average, scaled to some arbitary value
sub normalise
{
	my $self=shift;
	my $normal=shift;
	my $start=shift;
	my $stop=shift;
	return int( $self->sum($start,$stop)/($stop+1-$start)*$normal );
}

# Set this value
sub setmaxperhour($$)
{
	my $self=shift;
	my $maxperhour = shift;
	$self->{maxperhour} = $maxperhour;
	$self->{day}->setmaxperhour($maxperhour);
	$self->{week}->setmaxperhour($maxperhour);
	$self->{month}->setmaxperhour($maxperhour);
	$self->{year}->setmaxperhour($maxperhour);
}

sub cap_to_max_per_hour($$)
{
	my $self=shift;
	my $i=shift;
	if (defined $self->{maxperhour}) {
		my $maxperhour = $self->{maxperhour};
		if ($self->{value}[$i] > $self->{max}[$i]) {
			$self->{value}[$i] = $self->{max}[$i];
		}
	}
}

# Sum of usage from [$start to $stop]
# these are in terms of buckets, ie. time from origin
# for the last 60 seconds, you want sum(0,59)
# setvalues(start,stop,total,boolean:retain-original)
# start and stop are inclusive values (ie. 0,0 is fine)
sub setvalues($$$$)
{
	my $self=shift;

	my $value=shift;
	my $a=shift;
	my $b=shift;

	foreach my $t ( $self->{day}, $self->{week}, $self->{month}, $self->{year}) {
		$t->setvalues($value,$a,$b)
	}
}

sub settime($$)
{
	my $self =shift;
	$self->{timestamp}=shift;

}

sub showtime($)
{
	my $s = shift;
	if ($s<120) { return $s; }
	elsif ($s<3600) { return int($s/60)."m"; }
	elsif ($s<86400) { return int($s/3600)."h"; }
	return sprintf "%.0fd", ($s/86400);
}

sub print
{
	my $self=shift;
	my $start=shift;
	my $stop=shift;
	my ($i);
	my $timestamp = $self->{timestamp};
	for ($i=0; $i < $self->{maxoffset} ; $i++) {
		my $bucketstart = $self->{offset}[$i];
		my $bucketstop = $self->{offset}[$i]+$self->{duration}[$i]-1;
		if ((!defined $start || $start <= $bucketstart) &&
		    (!defined $stop  ||  $bucketstop <= $stop)) {
		    	my $n = $self->{value}[$i] / $self->{duration}[$i];
			if ($n>=0) {
				$n=log($n+1)/log(2)+1;
			}
			my $t=$self->{lasttime}[$i];
			printf "%4s+%-4s @%5s: %8.1f  %s\n",
				showtime($self->{offset}[$i]),
				showtime($self->{duration}[$i]),
				$t==0 ? "--" : $self->{timestamp}-$t,
				$self->{value}[$i],
				"X"x$n;
		}
	}
	print "\n";
}

1;

==> squish-0.0.14.tar.gz: squish-0.0.14/rrdsquish.pm <==
package shiftarray;

no warnings "recursion";
	# setvalue over limit calls setvalue recursively

##################################################
## the object constructor (simplistic version)  ##
##################################################
sub new
{
	#my $class = shift;
	my $interval = shift;
	my $samples = shift;

	my $self = {};
	if (defined $class) {
		bless($self, $class);
	}
	else {
		bless($self);
	}

	# Length of a single sample period (e.g. 5 mins)
	$self->{interval} = $interval;	# 5 minute samples 
	
	# Number of samples to make up the period
	$self->{samples} = $samples;		

	# How long is this graph
	$self->{period} = $interval * $samples;

	# Where we start logging in the data
	$self->{offset} = 0;			
	# The data itself (nothing yet)
	$self->{data} = ();	
	
	# Never been touched
	$self->{lasttime} = 0;			

	return $self;
}

sub dump($)
{
	my $self=shift;
	my $xx=0;
	my $i=0;
	my $p = " interval:".$self->{interval}.
		" samples:".$self->{samples}.
		" offset:".$self->{offset};

	while ($xx < $self->{period}) {
		$p .= "\ndata[$i]:".$self->{data}[$i];
		$i++;
		$xx+=$self->{interval};
	}
	return "$p\n";;
}

sub getperiod($)
{
	my $self=shift;
	return $self->{period};
}

sub getinterval($)
{
	my $self=shift;
	return $self->{interval};
}

sub setmaxperhour($)
{
	my $self = shift;
	my $max = shift;
	$self->{maxvalue} = $max/3600 * $self->{interval};
}

# Set a bucket value
# setvalue(x,y)
sub setvalue($$$)
{
	my $self = shift;
	my $bucket = shift;
	my $value = shift;

	if ($bucket>=$self->{samples} || $bucket<0) { return; }
	my $index=($bucket + $self->{offset} + $self->{samples}*4) % $self->{samples};

	if (defined $self->{maxvalue}) {
		if ($value < $self->{maxvalue}) {
			$self->{data}[$index]=$value;
		}
		else {
			my $max = $self->{maxvalue};
			$self->setvalue($bucket+1,$value-$max + $self->getvalue($bucket+1));
			$self->{data}[$index]=$max;
		}
	}
	else {
		$self->{data}[$index]=$value;
	}

	return $value;
}
# Get a bucket value
# y = getvalue(x);
sub getvalue($$)
{
	my $self = shift;
	my $bucket = shift;

	my $index=($bucket+$self->{offset}+$self->{samples}*4) % $self->{samples};
	my $value = $self->{data}[$index];
	if (defined $value) { return $value; } else { return 0; }
}

# Distribute $value from $a to $b
sub setvalues($$$$)
{
	my $self = shift;
	my $value = shift;
	my $a = shift;
	my $b = shift;


	my $period = $self->{period};
	my $interval = $self->{interval};
	# Maybe it's just one bucket?
	if ($a<0) {
		$value += $value / ($b-$a)*$a;
		$a=0;
	}
	if ($b < $a) { return; }
	if ($a>=$period) {
		return; # off the graph - we don't care
	}
	if ($b>$period) {
		$value -= $value / ($b-$a) * ($b-$period);
		$b=$period;
	}
	if ($b < $a) { return; }

#	# Speed -- All in the first bucket?
#	if ($a==0 && $b<=$self->{interval}) {
#		$value += $self->getvalue(0);
#		$self->setvalue(0,$value);
#		return;
#	}

	# Determine start bucket, end bucket and range betwixt
	my $first = $a / $interval;
	my $last = $b / $interval;
	my $unit;
	if ($last == $first) {
		$unit=$value;
	}
	else {
		$unit = $value / ($last-$first);  # how much is a whole bucket
	}

	# print "a:$a b:$b value:$value first:$first last:$last unit:$unit interval:$interval part:$part\n";
	$part=1-($first-int($first));
	for (my $j=int($first); $j<=$last && $value>0 ;$j++) {
		my $portion=$unit*$part;
		if ($portion>$value) { $portion=$value; }
		$self->setvalue($j, $portion + $self->getvalue($j));
		$value-=$portion;
		$part=1;
	}
}

# Move the origin a little
sub moveto($$)
{
	my $self = shift;
	my $time = shift;

	$seconds = $time - $self->{lasttime};
	if ($seconds <=0) {
		$self->{lasttime}=$time;
		return;  # can't go back in time, won't try
	}
	if ($seconds < $self->{interval}) {
		return;  # not enough to move me
	}
	if ($seconds > $self->{period}) {
		$self->{lasttime} = $time - $time % $self->{interval};
		$self->{data}=();   # all data has expired
		return;
	}
	my $shift = int($seconds / $self->{interval});
	my $samples = $self->{samples};
	$self->{offset} = ($self->{offset} - $shift + $samples) % ($samples);
	my $i;
	for ($i=0; $i<$shift; $i++) {
		$self->setvalue($i,0);
	}
	# $self->{lasttime} += $shift * $self->{interval};
	$self->{lasttime} = $time - $time % $self->{interval};
}

# Su0 of usage from 0 seconds to $stop seconds)
# these are in terms of buckets, ie. time from origin for the last 60 seconds,
# you want sum(60)
sub sum($$)
{
	my $self=shift;
	my $stop=shift;

	my $sum=0;
	my $n=($stop/$self->{interval});
	my $ni=int($n);

	for (my $i=0; $i<$ni;$i++) {
		$sum += $self->getvalue($i);
	}
	$sum+=($n-$ni)*$self->getvalue($ni);
}

sub total($)
{
	my $self=shift;
	my $a;
	my $total=0;
	my $t = $self->{data};

	if (defined $t) {
		foreach $a (@$t) {
			if (defined $a) { $total+=$a; }
		}
	}
	return $total;
}


package rrdsquish;
#

=head1 NAME

rrdsquish - A hacky rrd, with the dubious distinction that you can modify older
data

=head1 SYNOPSIS

Store a month worth of information in a rotating database

=head1 DESCRIPTION

To run this thingy you create B<rrdsquish::new> object.  Optionally, you might
use B<rrdsquish::setmaxperhour()> so set a cieling for an hour (if you're
counting seconds, setting 3600 per hour makes sense).

Once the rrdsquish is set up, you use it in the following fashion:

=over 8

=item B<rrdsquish::moveto(origin)>

moves the origin to the timestamp given

=item B<rrdsquish::setvalues(start,stop,value)>

smears the value over the database between start and stop (inclusive) (i.e a new log entry 
was processed, applicable from B<start> to B<stop>, having a value of B<value>.

=item B<rrdsquish::sum(start,stop)>

returns the total value from start to stop (inclusive)

=back


=head1 EXAMPLE

This should do something interesting.  It sets some (additive) values into the
graph at the current time, and then rotates the graph, printing it out bit by
bit.  It's a bit silly, but it's more interesting than painting a DNS server.

  use rrdsquish;
  
  $rrd = rrdsquish::new();
  $time=1000;
  $rrd->moveto($time);
  
  $rrd->setvalues(0,59,600);
  $rrd->setvalues(30,90,600);
  foreach $a (0..60) {
  	$rrd->print(20,240);
  	sleep 2;
  	$rrd->moveto($time+$a);
  }
  $rrd->print(25,240);
  print $rrd->sum();

=head1 SEE ALSO

rrdtool -- inspiration for this travesty of data manipulation.

=head1 BUGS

At the points where resolution is lost, it is possible to accumulate excess
baggage (you have to see it to know what I mean).  It looks kinda funny on a
graph.  This is (I reckon) a problem of matching a continous system to
a discrete system, and I leave it for a higher mind to solve.  It may help to
rotate in a more synchronised manner .. hey, let's try that!

=head1 AUTHOR

Andrew McGill <andrew...wh@tever...ledge.co.za>

=cut
#'  this is to make cperl.el happy
use strict;
#use Carp;
# use vars qw(@ISA $VERSION $maxperhour);
use vars qw($VERSION $maxperhour);

$VERSION = "1.01";

sub Version { $VERSION };

# Not inherited ...
# @ISA = qw(  );

# Maximum value that the DB will show per hour
# When you add values to the DB, this is set
my $maxperhour;

# Prototypes
sub setup;


##################################################
## the object constructor (simplistic version)  ##
##################################################
sub new
{
	my $class = shift;
	my $self = {};
	my %params = @_;
	if (defined $class) {
		bless($self, $class);
	}
	else {
		bless($self);
	}
	$self->setup();
	return $self;
}

# sets up a delay list or whatever we're calling this travesty of data
# rotation and loss
sub setup
{
	my $self=shift;
	
	$self->{day}   = shiftarray::new(300,   288 );
	$self->{week}  = shiftarray::new(1800,  336 );
	$self->{month} = shiftarray::new(7200,  366 );
	$self->{year}  = shiftarray::new(86400, 366 );

	$self->{timestamp}=1;
}

# Get data for making a graph of the last 5 minutes, 5 hours, 5 weeks, etc.
# 
# ($total,$max) = getgraph($interval,$duration,\@x,\@y)
# e.g. last 60 minutes in 30 second summaries
#	getgraph(30,600,\@x,\@y)
sub getgraph($$$$)
{
	my $self = shift;
	my $tt = shift;
	my $x = shift;  # array ref
	my $y = shift;  # array ref
	my $a;
	my $total = 0;
	my $max = 0;
	my $t;

	if ($tt==0)    { $t=$self->{day}   ; }
	elsif ($tt==1) { $t=$self->{week}  ; }
	elsif ($tt==2) { $t=$self->{month} ; }
	elsif ($tt==3) { $t=$self->{year}  ; }
	else { return; }

	my $interval = $t->getinterval();
	my $i=0;
	my $xx=0;
	while ($xx < $t->{period}) {
		my $this = $$y[$i]=$t->getvalue($i);
		$total+=$this;
		if ($this > $max) { $max=$this;}
		$$x[$i]=$xx;
		$i++;
		$xx+=$t->{interval};
	}
	return ($total,$max);
}

sub dump($)
{
	my $self = shift;

	return	"Day: "  .$self->{day}->dump().
		"Week: " .$self->{week}->dump().
		"Month: ".$self->{month}->dump().
		"Year: " .$self->{year}->dump() . "\n";
}

# Shift stuff, so that the $time given is the current time for the array
# e.g. move(time());
sub moveto
{
	my $self = shift;
	my $time = shift;
	# Go through db backwards - we're assumming a loss of
	# resolution from low indexes to high .. ie. seconds, minutes,
	# hours, days, months
	$self->{day}  ->moveto($time);
	$self->{week} ->moveto($time);
	$self->{month}->moveto($time);
	$self->{year} ->moveto($time);

	$self->{timestamp}=$time;
}

# Sum of usage from 0 to $stop (excluding $stoip)
# these are in terms of buckets, ie. time from origin
# for the last 60 seconds, you want sum(0,59)
sub sum($$)
{
	my $self=shift;
	my $stop=shift;

	if ($stop <= $self->{day}  ->getperiod()) { return $self->{day}  ->sum($stop); }
	if ($stop <= $self->{week} ->getperiod()) { return $self->{week} ->sum($stop); }
	if ($stop <= $self->{month}->getperiod()) { return $self->{month}->sum($stop); }
	                                            return $self->{year} ->sum($stop);
}

# Add up a day, week, month or year
sub total($$)
{
	my $self=shift;
	my $tt=shift;
	my $t;

	if ($tt==0)    { $t=$self->{day}   ; }
	elsif ($tt==1) { $t=$self->{week}  ; }
	elsif ($tt==2) { $t=$self->{month} ; }
	elsif ($tt==3) { $t=$self->{year}  ; }
	else { return; }
	return $t->total();
}

# Print an average, scaled to some arbitary value
sub normalise
{
	my $self=shift;
	my $normal=shift;
	my $start=shift;
	my $stop=shift;
	return int( $self->sum($start,$stop)/($stop+1-$start)*$normal );
}

# Set this value
sub setmaxperhour($$)
{
	my $self=shift;
	my $maxperhour = shift;
	$self->{maxperhour} = $maxperhour;
	$self->{day}->setmaxperhour($maxperhour);
	$self->{week}->setmaxperhour($maxperhour);
	$self->{month}->setmaxperhour($maxperhour);
	$self->{year}->setmaxperhour($maxperhour);
}

sub cap_to_max_per_hour($$)
{
	my $self=shift;
	my $i=shift;
	if (defined $self->{maxperhour}) {
		my $maxperhour = $self->{maxperhour};
		if ($self->{value}[$i] > $self->{max}[$i]) {
			$self->{value}[$i] = $self->{max}[$i];
		}
	}
}

# Sum of usage from [$start to $stop]
# these are in terms of buckets, ie. time from origin
# for the last 60 seconds, you want sum(0,59)
# setvalues(start,stop,total,boolean:retain-original)
# start and stop are inclusive values (ie. 0,0 is fine)
sub setvalues($$$$)
{
	my $self=shift;

	my $value=shift;
	my $a=shift;
	my $b=shift;

	foreach my $t ( $self->{day}, $self->{week}, $self->{month}, $self->{year}) {
		$t->setvalues($value,$a,$b)
	}
}

sub settime($$)
{
	my $self =shift;
	$self->{timestamp}=shift;

}

sub showtime($)
{
	my $s = shift;
	if ($s<120) { return $s; }
	elsif ($s<3600) { return int($s/60)."m"; }
	elsif ($s<86400) { return int($s/3600)."h"; }
	return sprintf "%.0fd", ($s/86400);
}

sub print
{
	my $self=shift;
	my $start=shift;
	my $stop=shift;
	my ($i);
	my $timestamp = $self->{timestamp};
	for ($i=0; $i < $self->{maxoffset} ; $i++) {
		my $bucketstart = $self->{offset}[$i];
		my $bucketstop = $self->{offset}[$i]+$self->{duration}[$i]-1;
		if ((!defined $start || $start <= $bucketstart) &&
		    (!defined $stop  ||  $bucketstop <= $stop)) {
		    	my $n = $self->{value}[$i] / $self->{duration}[$i];
			if ($n>=0) {
				$n=log($n+1)/log(2)+1;
			}
			my $t=$self->{lasttime}[$i];
			printf "%4s+%-4s @%5s: %8.1f  %s\n",
				showtime($self->{offset}[$i]),
				showtime($self->{duration}[$i]),
				$t==0 ? "--" : $self->{timestamp}-$t,
				$self->{value}[$i],
				"X"x$n;
		}
	}
	print "\n";
}

1;
